@@ -61,6 +61,8 @@ func (s *BaseSnapSuite) readPassword(fd int) ([]byte, error) {
 
 func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
+	dirs.SetRootDir(c.MkDir())
+
 	s.stdin = bytes.NewBuffer(nil)
 	s.stdout = bytes.NewBuffer(nil)
 	s.stderr = bytes.NewBuffer(nil)
@@ -83,6 +85,7 @@ func (s *BaseSnapSuite) TearDownTest(c *C) {
 	c.Assert(s.AuthFile == "", Equals, false)
 	err := os.Unsetenv(TestAuthFileEnvKey)
 	c.Assert(err, IsNil)
+	dirs.SetRootDir("/")
 	s.BaseTest.TearDownTest(c)
 }
 
@@ -128,6 +131,7 @@ var _ = Suite(&SnapSuite{})
 func DecodedRequestBody(c *C, r *http.Request) map[string]interface{} {
 	var body map[string]interface{}
 	decoder := json.NewDecoder(r.Body)
+	decoder.UseNumber()
 	err := decoder.Decode(&body)
 	c.Assert(err, IsNil)
 	return body
@@ -152,9 +156,9 @@ func mockVersion(v string) (restore func()) {
 	return func() { cmd.Version = old }
 }
 
-func mockSnapConfine() func() {
-	snapConfine := filepath.Join(dirs.DistroLibExecDir, "snap-confine")
-	if err := os.MkdirAll(dirs.DistroLibExecDir, 0755); err != nil {
+func mockSnapConfine(libExecDir string) func() {
+	snapConfine := filepath.Join(libExecDir, "snap-confine")
+	if err := os.MkdirAll(libExecDir, 0755); err != nil {
 		panic(err)
 	}
 	if err := ioutil.WriteFile(snapConfine, nil, 0644); err != nil {
@@ -240,9 +244,6 @@ func (s *SnapSuite) TestUnknownCommand(c *C) {
 }
 
 func (s *SnapSuite) TestResolveApp(c *C) {
-	dirs.SetRootDir(c.MkDir())
-	defer dirs.SetRootDir("/")
-
 	err := os.MkdirAll(dirs.SnapBinariesDir, 0755)
 	c.Assert(err, IsNil)
 
