@@ -36,6 +36,7 @@ const networkControlConnectedPlugAppArmor = `
 # trusted apps.
 
 #include <abstractions/nameservice>
+/run/systemd/resolve/stub-resolv.conf r,
 
 # systemd-resolved (not yet included in nameservice abstraction)
 #
@@ -249,6 +250,13 @@ socket AF_NETLINK - NETLINK_GENERIC
 socket AF_NETLINK - NETLINK_KOBJECT_UEVENT
 `
 
+const networkControlConnectedPlugUDev = `
+KERNEL=="rfkill",    TAG+="###CONNECTED_SECURITY_TAGS###"
+KERNEL=="tap[0-9]*", TAG+="###CONNECTED_SECURITY_TAGS###"
+KERNEL=="tun",       TAG+="###CONNECTED_SECURITY_TAGS###"
+KERNEL=="tun[0-9]*", TAG+="###CONNECTED_SECURITY_TAGS###"
+`
+
 func init() {
 	registerIface(&commonInterface{
 		name:                  "network-control",
@@ -258,6 +266,8 @@ func init() {
 		baseDeclarationSlots:  networkControlBaseDeclarationSlots,
 		connectedPlugAppArmor: networkControlConnectedPlugAppArmor,
 		connectedPlugSecComp:  networkControlConnectedPlugSecComp,
+		connectedPlugUDev:     networkControlConnectedPlugUDev,
 		reservedForOS:         true,
 	})
+
 }
